/*
  Copyright 2020 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.talk

import com.kakao.sdk.partner.Constants
import com.kakao.sdk.partner.talk.model.*
import com.kakao.sdk.talk.model.FriendOrder
import com.kakao.sdk.talk.model.Friends
import com.kakao.sdk.talk.model.Order
import com.kakao.sdk.template.model.DefaultTemplate
import retrofit2.Call
import retrofit2.http.*

/**
 * @suppress
 */
interface PartnerTalkApi {
    @GET(Constants.V1_FRIENDS_PATH)
    fun friends(
            @Query(Constants.FRIEND_TYPE) friendType: FriendType? = null,
            @Query(Constants.FRIEND_FILTER) friendFilter: FriendFilter? = null,
            @Query(com.kakao.sdk.talk.Constants.FRIEND_ORDER) friendOrder: FriendOrder? = null,
            @Query(com.kakao.sdk.talk.Constants.OFFSET) offset: Int? = null,
            @Query(com.kakao.sdk.talk.Constants.LIMIT) limit: Int? = null,
            @Query(com.kakao.sdk.talk.Constants.ORDER) order: Order? = null,
            @Query(com.kakao.sdk.talk.Constants.COUNTRY_CODES) countryCodes: String? = null
    ): Call<Friends<PartnerFriend>>

    @GET(Constants.V1_CHATS_PATH)
    fun chatList(
        @Query(Constants.FILTER) filter: String? = null,
        @Query(Constants.OFFSET) offset: Int? = null,
        @Query(com.kakao.sdk.talk.Constants.LIMIT) limit: Int? = null,
        @Query(com.kakao.sdk.talk.Constants.ORDER) order: Order? = null,
        @Query(com.kakao.sdk.talk.Constants.SECURE_RESOURCE) secureResource: Boolean? = true
    ): Call<Chats>

    @GET(Constants.V1_CHAT_MEMBERS_PATH)
    fun chatMembers(
        @Query(Constants.CHAT_ID) chatId: Long,
        @Query(Constants.FRIENDS_ONLY) friendsOnly: Boolean? = null,
        @Query(Constants.INCLUDE_PROFILE) includeProfile: Boolean? = null,
        @Query(com.kakao.sdk.talk.Constants.OFFSET) offset: Int? = null,
        @Query(com.kakao.sdk.talk.Constants.LIMIT) limit: Int? = null,
        @Query(com.kakao.sdk.talk.Constants.ORDER) order: Order? = null,
        @Query(com.kakao.sdk.talk.Constants.SECURE_RESOURCE) secureResource: Boolean? = true
    ): Call<ChatMembers>

    @FormUrlEncoded
    @POST(Constants.V2_MESSAGE_PATH)
    fun sendMessage(
        @Field(Constants.RECEIVER_ID_TYPE) receiverIdType: String? = null,
        @Field(Constants.RECEIVER_IDS) receiverIds: String,
        @Field(com.kakao.sdk.talk.Constants.TEMPLATE_ID) templateId: Long,
        @Field(com.kakao.sdk.talk.Constants.TEMPLATE_ARGS) templateArgs: Map<String, String>? = null
    ): Call<PartnerMessageSendResult>

    @FormUrlEncoded
    @POST(Constants.V2_MESSAGE_DEFAULT_PATH)
    fun sendMessage(
        @Field(Constants.RECEIVER_ID_TYPE) receiverIdType: String? = null,
        @Field(Constants.RECEIVER_IDS) receiverIds: String,
        @Field(com.kakao.sdk.talk.Constants.TEMPLATE_OBJECT) template: DefaultTemplate
    ): Call<PartnerMessageSendResult>

    @FormUrlEncoded
    @POST(Constants.V2_MESSAGE_SCRAP_PATH)
    fun sendMessage(
        @Field(Constants.RECEIVER_ID_TYPE) receiverIdType: String? = null,
        @Field(Constants.RECEIVER_IDS) receiverIds: String,
        @Field(com.kakao.sdk.talk.Constants.REQUEST_URL) requestUrl: String,
        @Field(com.kakao.sdk.talk.Constants.TEMPLATE_ID) templateId: Long? = null,
        @Field(com.kakao.sdk.talk.Constants.TEMPLATE_ARGS) templateArgs: Map<String, String>? = null
    ): Call<PartnerMessageSendResult>
}